<?php

if ( ! class_exists( 'SLP_Plugin_Style_Manager' ) ):

	/**
	 * class SLP_Plugin_Style_Manager
	 *
	 * @package StoreLocatorPlus\PluginStyle\Manager
	 * @author Lance Cleveland <lance@charlestonsw.com>
	 * @copyright 2012-2016 Charleston Software Associates, LLC	 *
	 *
	 * @var             SLP_Plugin_Style        $plugin_style
	 */
	class SLP_Plugin_Style_Manager extends SLPlus_BaseClass_Object {
	    private $plugin_style;

		/**
		 * Add the theme settings to the admin panel.
		 *
		 * @param   SLP_Settings    $settings
		 * @param   string          $section_name
		 * @param   string          $group_name
		 */
		public function add_settings($settings ,$section_name ,$group_name ) {
			$this->create_object_plugin_style();
			$theme_list = $this->plugin_style->get_theme_list();

			$settings->add_ItemToGroup(array(
				'label'         => __( 'Plugin Style' , 'store-locator-le' ),
				'use_prefix'    => false,
				'setting'       => 'theme'                               ,
				'value'         => $this->slplus->options_nojs['theme']         ,
				'type'          => 'list'                                       ,
				'section'       => $section_name                                ,
				'group'         => $group_name                                  ,
				'custom'        => $theme_list                     ,
			));

			$settings->add_ItemToGroup(array(
				'type'        => 'subheader'                                ,
				'description' =>
					__( 'Select a plugin style to change the CSS styling and layout of the slplus shortcode elements. ' , 'store-locator-le' ) .
					__( 'This determines how the locator looks within your page. ' , 'store-locator-le' ) .
					'<span class="new_paragraph">'.
					__( 'Most plugin styles work best with the following add ons installed: ' , 'store-locator-le' ) .
					'<br/>' .
					$this->slplus->text_manager->get_web_link( 'shop_for_experience' )  .
					'<br/>' .
					$this->slplus->text_manager->get_web_link( 'shop_for_premier' ) .
					'</span>' .
					$this->slplus->text_manager->get_web_link( 'docs_for_plugin_style' )
			,
				'section'     => ( is_null( $section_name  ) ?  __( 'Display Settings' , 'store-locator-le' ) : $section_name )                      ,
				'group'       => $group_name                                ,
				'show_label'   => false
			));

			// Add Style Details Divs
			//
			$settings->add_ItemToGroup(array(
				'label'         => '',
				'description'   => $this->plugin_style->setup_ThemeDetails( $theme_list ),
				'section'       => $section_name     ,
				'group'         => $group_name       ,
				'setting'       => 'themedesc'  ,
				'type'          => 'subheader'  ,
				'show_label'    => false
			));
		}

	    /**
	     * Attach plugin_style object to this->plugin_style.
	     */
	    private function create_object_plugin_style() {
	        if ( ! isset( $this->plugin_style ) ) {
		        require_once( SLPLUS_PLUGINDIR . 'include/unit/SLP_Plugin_Style.php' );
	            $this->plugin_style = new SLP_Plugin_Style();
	        }
	    }

	}

endif;