<?php

if ( !class_exists( 'SLP_Option' ) ) :

	require_once( SLPLUS_PLUGINDIR . 'include/base_class.object.php' );

    /**
     * Class SLP_Option
     *
     * A smarter SLPlus option object.
     *
     * @package   StoreLocatorPlus\Option
     * @author    Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2016 Charleston Software Associates, LLC
     *
     * @property        null||callback  $call_when_changed    the function to call when the value changes - gets $key, $old_value, $new_value as parameters - executed with ->execute_change_callbacks
     * @property        mixed           $default_if_empty     default value if empty
     * @property-read   mixed           $initial_value        the initial value from the SLP options
     * @property        boolean         $is_text              true if a gettext translatable string
     * @property        string          $slug                 the slug
     * @property        boolean         $use_in_javascript    use in javascript (true: gets localized in localize_script)
     * @property-read    mixed           $value                current value of option
     *
     * @since     4.5.09
     */
    class SLP_Option extends SLPlus_BaseClass_Object {
    	public $call_when_changed = null;
        public $default_if_empty = '';
        public $is_text = false;
        public $slug;
        public $use_in_javascript = false;
	    private $initial_value;
	    private $value;


	    /**
	     * Get the value, running it through a filter.
	     *
	     * @param string $property
	     *
	     * @return mixed     null if not set or the value
	     */
	    function __get( $property ) {
		    if ( property_exists( $this, $property ) ) {

		    	if ( $property === 'value' ) {

				    // Not Set Yet: Set value of smart option from SLP options or options_nojs
				    //
				    if ( ! isset( $this->$property->initial_value ) ) {
					    if ( $this->use_in_javascript ) {
						    $this->initial_value = array_key_exists( $this->slug, $this->slplus->options ) ? $this->slplus->options[ $this->slug ] : null;
					    } else {
						    $this->initial_value = array_key_exists( $this->slug, $this->slplus->options_nojs ) ? $this->slplus->options_nojs[ $this->slug ] : null;
					    }
				    }

				    // Default If Empty
				    //
				    if ( empty( $this->initial_value ) ) {
					    $this->value = $this->default_if_empty;
				    }  else {
					    $this->value = $this->initial_value;
				    }

				    /**
				     * FILTER: slp_get_option_<property>
				     *
				     * <property> can be a slplus->options or slplus->options_nojs array key
				     */
				    $this->value =  apply_filters( 'slp_get_option_' . $this->slug, $this->value );

			    }

			    return $this->$property;
		    }

		    return null;
	    }

	    /**
	     * Allow isset to be called on private properties.
	     *
	     * @param $property
	     *
	     * @return bool
	     */
	    public function __isset( $property ) {

		    // Always set value. It can't be not set or empty() always fails with the slp_get_option_<slug> filter.
	    	if ( $property === 'value' ) {
	    		$this->__get( 'value' );
		    }

		    return isset( $this->$property );
	    }

	    /**
	     * Allow value to be set directly.
	     *
	     * @param $property
	     *
	     * @param $value
	     * @return SLP_Option
	     */
	    public function __set( $property, $value ) {
		    if ( property_exists( $this, $property ) ) {
		    	if ( $property === 'value' ) {
				    $this->$property = $value;
			    }
			    return $this;
		    }
	    }


    }

endif;